"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.optional = exports.union = exports.intersection = exports.codec = void 0;
const defs = require("../definitions");
const utils = require("../utils");
const codec = (tag, encode, decode, props) => {
    const c = {
        _tag: tag,
        props: {
            ...(props || {}),
            metadata: props?.metadata ?? {},
            required: props?.required ?? true
        },
        encode,
        decode,
        and: (extention) => (0, exports.intersection)(c, extention),
        or: (extention) => (0, exports.union)(c, extention),
        meta: (metadata) => {
            return (0, exports.codec)(tag, encode, decode, {
                ...c.props,
                metadata: {
                    ...c.props.metadata,
                    ...metadata
                }
            });
        },
        optional: () => (0, exports.optional)(c)
    };
    return c;
};
exports.codec = codec;
const mergeSameCodecs = (tag, c1, c2) => {
    const codecs = [];
    if (utils.isCodecType(c1, tag)) {
        codecs.push(...c1.props.codecs);
    }
    else {
        codecs.push(c1);
    }
    if (utils.isCodecType(c2, tag)) {
        codecs.push(...c2.props.codecs);
    }
    else {
        codecs.push(c2);
    }
    return codecs;
};
const intersection = (c1, c2) => {
    const codecs = mergeSameCodecs(defs.CodecType.Intersection, c1, c2);
    const transformer = (transformer) => (data) => {
        return codecs.reduce((acc, codec) => {
            return {
                ...acc,
                ...codec[transformer](data)
            };
        }, {});
    };
    return (0, exports.codec)(defs.CodecType.Intersection, transformer('encode'), transformer('decode'), {
        codecs: codecs
    });
};
exports.intersection = intersection;
const union = (c1, c2) => {
    const codecs = mergeSameCodecs(defs.CodecType.Union, c1, c2);
    const transformer = (transformer) => (data) => {
        const errors = [];
        for (const codec of codecs) {
            try {
                return codec[transformer](data);
            }
            catch (err) {
                errors.push(err);
            }
        }
        throw new utils.TransformError(errors
            .map((error) => {
            if (error instanceof utils.TransformError) {
                return error.errors;
            }
            return error.toString();
        })
            .flat());
    };
    return (0, exports.codec)(defs.CodecType.Union, transformer('encode'), transformer('decode'), {
        codecs
    });
};
exports.union = union;
const optional = (type) => {
    return (0, exports.codec)(type._tag, (data) => {
        if (data === undefined) {
            return undefined;
        }
        return type.encode(data);
    }, (data) => {
        if (data === undefined) {
            return undefined;
        }
        return type.decode(data);
    }, {
        ...type.props,
        required: false
    });
};
exports.optional = optional;
//# sourceMappingURL=codec.js.map